==> codeready-workspaces-2.7.0-GA-crwctl3-sources.tar.gz: crwctl/lib/util.js <==
"use strict";
/*********************************************************************
 * Copyright (c) 2020 Red Hat, Inc.
 *
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 **********************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
exports.findWorkingNamespace = exports.getEmbeddedTemplatesDirectory = exports.downloadYaml = exports.downloadFile = exports.safeSaveYamlToFile = exports.safeLoadFromYamlFile = exports.readPackageJson = exports.getProjectName = exports.getProjectVersion = exports.isDirEmpty = exports.askForChectlUpdateIfNeeded = exports.notifyCommandCompletedSuccessfully = exports.getCommandErrorMessage = exports.getCommandSuccessMessage = exports.readCRFile = exports.sleep = exports.getImageTag = exports.base64Decode = exports.generatePassword = exports.isOpenshiftPlatformFamily = exports.isKubernetesPlatformFamily = exports.getClusterClientCommand = exports.OPENSHIFT_CLI = exports.KUBERNETES_CLI = void 0;
const tslib_1 = require("tslib");
const axios_1 = require("axios");
const cli_ux_1 = require("cli-ux");
const commandExists = require("command-exists");
const fs = require("fs-extra");
const https = require("https");
const yaml = require("js-yaml");
const notifier = require("node-notifier");
const path = require("path");
const pkjson = require('../package.json');
const context_1 = require("./api/context");
const kube_1 = require("./api/kube");
const version_1 = require("./api/version");
const constants_1 = require("./constants");
exports.KUBERNETES_CLI = 'kubectl';
exports.OPENSHIFT_CLI = 'oc';
function getClusterClientCommand() {
    const clusterClients = [exports.KUBERNETES_CLI, exports.OPENSHIFT_CLI];
    for (const command of clusterClients) {
        if (commandExists.sync(command)) {
            return command;
        }
    }
    throw new Error('No cluster CLI client is installed.');
}
exports.getClusterClientCommand = getClusterClientCommand;
function isKubernetesPlatformFamily(platform) {
    return platform === 'k8s' || platform === 'minikube' || platform === 'microk8s';
}
exports.isKubernetesPlatformFamily = isKubernetesPlatformFamily;
function isOpenshiftPlatformFamily(platform) {
    return platform === 'openshift' || platform === 'minishift' || platform === 'crc';
}
exports.isOpenshiftPlatformFamily = isOpenshiftPlatformFamily;
function generatePassword(passwodLength, charactersSet = '') {
    let dictionary;
    if (!charactersSet) {
        const ZERO_CHAR_CODE = 48;
        const NINE_CHAR_CODE = 57;
        const A_CHAR_CODE = 65;
        const Z_CHAR_CODE = 90;
        const a_CHAR_CODE = 97;
        const z_CHAR_CODE = 122;
        const ranges = [[ZERO_CHAR_CODE, NINE_CHAR_CODE], [A_CHAR_CODE, Z_CHAR_CODE], [a_CHAR_CODE, z_CHAR_CODE]];
        dictionary = [];
        for (let range of ranges) {
            for (let charCode = range[0]; charCode <= range[1]; charCode++) {
                dictionary.push(String.fromCharCode(charCode));
            }
        }
    }
    else {
        dictionary = [...charactersSet];
    }
    let generatedPassword = '';
    for (let i = 0; i < passwodLength; i++) {
        const randomIndex = Math.floor(Math.random() * dictionary.length);
        generatedPassword += dictionary[randomIndex];
    }
    return generatedPassword;
}
exports.generatePassword = generatePassword;
function base64Decode(arg) {
    return Buffer.from(arg, 'base64').toString('ascii');
}
exports.base64Decode = base64Decode;
/**
 * Returns the tag of the image.
 */
function getImageTag(image) {
    let entries = image.split('@');
    if (entries.length === 2) {
        // digest
        return entries[1];
    }
    entries = image.split(':');
    // tag
    return entries[1];
}
exports.getImageTag = getImageTag;
function sleep(ms) {
    // tslint:disable-next-line no-string-based-set-timeout
    return new Promise(resolve => setTimeout(resolve, ms));
}
exports.sleep = sleep;
/**
 * Returns CR file content. Throws an error, if file doesn't exist.
 * @param flags - parent command flags
 * @param CRKey - key for CR file flag
 * @param command - parent command
 */
function readCRFile(flags, CRKey) {
    const CRFilePath = flags[CRKey];
    if (!CRFilePath) {
        return;
    }
    if (fs.existsSync(CRFilePath)) {
        return safeLoadFromYamlFile(CRFilePath);
    }
    throw new Error(`Unable to find file defined in the flag '--${CRKey}'`);
}
exports.readCRFile = readCRFile;
/**
 * Returns command success message with execution time.
 */
function getCommandSuccessMessage() {
    const ctx = context_1.ChectlContext.get();
    if (ctx[context_1.ChectlContext.START_TIME]) {
        if (!ctx[context_1.ChectlContext.END_TIME]) {
            ctx[context_1.ChectlContext.END_TIME] = Date.now();
        }
        const workingTimeInSeconds = Math.round((ctx[context_1.ChectlContext.END_TIME] - ctx[context_1.ChectlContext.START_TIME]) / 1000);
        const minutes = Math.floor(workingTimeInSeconds / 60);
        const seconds = (workingTimeInSeconds - minutes * 60) % 60;
        const minutesToStr = minutes.toLocaleString([], { minimumIntegerDigits: 2 });
        const secondsToStr = seconds.toLocaleString([], { minimumIntegerDigits: 2 });
        return `Command ${ctx[context_1.ChectlContext.COMMAND_ID]} has completed successfully in ${minutesToStr}:${secondsToStr}.`;
    }
    return `Command ${ctx[context_1.ChectlContext.COMMAND_ID]} has completed successfully.`;
}
exports.getCommandSuccessMessage = getCommandSuccessMessage;
/**
 * Returns command error message.
 */
function getCommandErrorMessage(err) {
    const ctx = context_1.ChectlContext.get();
    const logDirectory = ctx[context_1.ChectlContext.LOGS_DIR];
    let message = `${err}\nCommand ${ctx[context_1.ChectlContext.COMMAND_ID]} failed. Error log: ${ctx[context_1.ChectlContext.ERROR_LOG]}`;
    if (logDirectory && isDirEmpty(logDirectory)) {
        message += ` CodeReady Workspaces logs: ${logDirectory}`;
    }
    return message;
}
exports.getCommandErrorMessage = getCommandErrorMessage;
function notifyCommandCompletedSuccessfully() {
    notifier.notify({
        title: 'crwctl',
        message: getCommandSuccessMessage()
    });
}
exports.notifyCommandCompletedSuccessfully = notifyCommandCompletedSuccessfully;
function askForChectlUpdateIfNeeded() {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const ctx = context_1.ChectlContext.get();
        if (yield version_1.VersionHelper.isChectlUpdateAvailable(ctx[context_1.ChectlContext.CACHE_DIR])) {
            cli_ux_1.cli.info('A newer version of crwctl is available.');
            if (yield cli_ux_1.cli.confirm('To deploy the latest version of CodeReady Workspaces you have to update crwctl first [y/n]')) {
                cli_ux_1.cli.info('Please run "crwctl update" and then repeat "server:deploy" command.');
                cli_ux_1.cli.exit(0);
            }
        }
    });
}
exports.askForChectlUpdateIfNeeded = askForChectlUpdateIfNeeded;
/**
 * Determine if a directory is empty.
 */
function isDirEmpty(dirname) {
    try {
        return fs.readdirSync(dirname).length === 0;
        // Fails in case if directory doesn't exist
    }
    catch (_a) {
        return true;
    }
}
exports.isDirEmpty = isDirEmpty;
/**
 * Returns current crwctl version defined in package.json.
 */
function getProjectVersion() {
    return pkjson.version;
}
exports.getProjectVersion = getProjectVersion;
/**
 * Returns current crwctl version defined in package.json.
 */
function getProjectName() {
    return pkjson.name;
}
exports.getProjectName = getProjectName;
function readPackageJson() {
    return JSON.parse(fs.readFileSync('../package.json').toString());
}
exports.readPackageJson = readPackageJson;
function safeLoadFromYamlFile(filePath) {
    return yaml.safeLoad(fs.readFileSync(filePath).toString());
}
exports.safeLoadFromYamlFile = safeLoadFromYamlFile;
function safeSaveYamlToFile(yamlObject, filePath) {
    fs.writeFileSync(filePath, yaml.safeDump(yamlObject));
}
exports.safeSaveYamlToFile = safeSaveYamlToFile;
function downloadFile(url, dest) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const streamWriter = fs.createWriteStream(dest);
        const response = yield axios_1.default({ url, method: 'GET', responseType: 'stream' });
        response.data.pipe(streamWriter);
        return new Promise((resolve, reject) => {
            streamWriter.on('finish', resolve);
            streamWriter.on('error', reject);
        });
    });
}
exports.downloadFile = downloadFile;
/**
 * Downloads yaml file and returns data converted to JSON.
 * @param url link to yaml file
 */
function downloadYaml(url) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const axiosInstance = axios_1.default.create({
            httpsAgent: new https.Agent({})
        });
        const response = yield axiosInstance.get(url);
        return yaml.safeLoad(response.data);
    });
}
exports.downloadYaml = downloadYaml;
function getEmbeddedTemplatesDirectory() {
    // Embedded templates are located in the templates directory that is in the project/installation root:
    // crwctl
    //  |- templates
    //  |- src
    //  |   |- util.ts
    //  |  ...
    //  |- lib
    //  |   |- util.js
    // ... ...
    // __dirname is
    //   project_root/src if dev mode,
    //   installation_root/lib if run from an installed location
    return path.join(__dirname, '..', 'templates');
}
exports.getEmbeddedTemplatesDirectory = getEmbeddedTemplatesDirectory;
/**
 * The default CodeReady Workspaces namespace has been changed from 'che' to 'eclipse-che'.
 * It checks if legacy namespace 'che' exists. If so all crwctl commands
 * will launched against that namespace otherwise default 'eclipse-che' namespace will be used.
 */
function findWorkingNamespace(flags) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (flags.chenamespace) {
            // use user specified namespace
            return flags.chenamespace;
        }
        const kubeHelper = new kube_1.KubeHelper(flags);
        if (yield kubeHelper.getNamespace(constants_1.DEFAULT_CHE_NAMESPACE)) {
            return constants_1.DEFAULT_CHE_NAMESPACE;
        }
        if (yield kubeHelper.getNamespace(constants_1.LEGACY_CHE_NAMESPACE)) {
            return constants_1.LEGACY_CHE_NAMESPACE;
        }
        return constants_1.DEFAULT_CHE_NAMESPACE;
    });
}
exports.findWorkingNamespace = findWorkingNamespace;
//# sourceMappingURL=util.js.map

==> codeready-workspaces-2.12.1-GA-crwctl3-sources.tar.gz: crwctl/lib/util.js <==
"use strict";
/**
 * Copyright (c) 2019-2021 Red Hat, Inc.
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *   Red Hat, Inc. - initial API and implementation
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.addTrailingSlash = exports.getDistribution = exports.findWorkingNamespace = exports.getEmbeddedTemplatesDirectory = exports.downloadYaml = exports.downloadFile = exports.safeSaveYamlToFile = exports.safeLoadFromYamlFile = exports.readPackageJson = exports.getProjectName = exports.getProjectVersion = exports.isDirEmpty = exports.askForChectlUpdateIfNeeded = exports.notifyCommandCompletedSuccessfully = exports.newError = exports.wrapCommandError = exports.getCommandSuccessMessage = exports.readCRFile = exports.sleep = exports.getImageTag = exports.getImageNameAndTag = exports.base64Decode = exports.generatePassword = exports.isOpenshiftPlatformFamily = exports.isKubernetesPlatformFamily = exports.getClusterClientCommand = exports.OPENSHIFT_CLI = exports.KUBERNETES_CLI = void 0;
const tslib_1 = require("tslib");
const update_1 = require("@oclif/plugin-update/lib/commands/update");
const axios_1 = require("axios");
const cli_ux_1 = require("cli-ux");
const commandExists = require("command-exists");
const fs = require("fs-extra");
const getos = require("getos");
const https = require("https");
const yaml = require("js-yaml");
const notifier = require("node-notifier");
const os = require("os");
const path = require("path");
const util_1 = require("util");
const context_1 = require("./api/context");
const kube_1 = require("./api/kube");
const version_1 = require("./api/version");
const constants_1 = require("./constants");
const pkjson = require('../package.json');
exports.KUBERNETES_CLI = 'kubectl';
exports.OPENSHIFT_CLI = 'oc';
function getClusterClientCommand() {
    const clusterClients = [exports.KUBERNETES_CLI, exports.OPENSHIFT_CLI];
    for (const command of clusterClients) {
        if (commandExists.sync(command)) {
            return command;
        }
    }
    throw new Error('No cluster CLI client is installed.');
}
exports.getClusterClientCommand = getClusterClientCommand;
function isKubernetesPlatformFamily(platform) {
    return platform === 'k8s' || platform === 'minikube' || platform === 'microk8s';
}
exports.isKubernetesPlatformFamily = isKubernetesPlatformFamily;
function isOpenshiftPlatformFamily(platform) {
    return platform === 'openshift' || platform === 'minishift' || platform === 'crc';
}
exports.isOpenshiftPlatformFamily = isOpenshiftPlatformFamily;
function generatePassword(passwodLength, charactersSet = '') {
    let dictionary;
    if (!charactersSet) {
        const ZERO_CHAR_CODE = 48;
        const NINE_CHAR_CODE = 57;
        const A_CHAR_CODE = 65;
        const Z_CHAR_CODE = 90;
        const a_CHAR_CODE = 97;
        const z_CHAR_CODE = 122;
        const ranges = [[ZERO_CHAR_CODE, NINE_CHAR_CODE], [A_CHAR_CODE, Z_CHAR_CODE], [a_CHAR_CODE, z_CHAR_CODE]];
        dictionary = [];
        for (const range of ranges) {
            for (let charCode = range[0]; charCode <= range[1]; charCode++) {
                dictionary.push(String.fromCharCode(charCode));
            }
        }
    }
    else {
        dictionary = [...charactersSet];
    }
    let generatedPassword = '';
    for (let i = 0; i < passwodLength; i++) {
        const randomIndex = Math.floor(Math.random() * dictionary.length);
        generatedPassword += dictionary[randomIndex];
    }
    return generatedPassword;
}
exports.generatePassword = generatePassword;
function base64Decode(arg) {
    return Buffer.from(arg, 'base64').toString('ascii');
}
exports.base64Decode = base64Decode;
/**
 * Separates docker image repository and tag.
 * @param image string with image and tag separated by a colon
 * @returns image name (including registry and account) and image tag correspondingly
 */
function getImageNameAndTag(image) {
    let deployedCheOperatorImageName;
    let deployedCheOperatorImageTag;
    if (image.includes('@')) {
        // Image is referenced via a digest
        const index = image.indexOf('@');
        deployedCheOperatorImageName = image.substring(0, index);
        deployedCheOperatorImageTag = image.substring(index + 1);
    }
    else {
        // Image is referenced via a tag
        const lastColonIndex = image.lastIndexOf(':');
        if (lastColonIndex === -1) {
            // Image name without a tag
            deployedCheOperatorImageName = image;
            deployedCheOperatorImageTag = 'latest';
        }
        else {
            const beforeLastColon = image.substring(0, lastColonIndex);
            const afterLastColon = image.substring(lastColonIndex + 1);
            if (afterLastColon.includes('/')) {
                // The colon is for registry port and not for a tag
                deployedCheOperatorImageName = image;
                deployedCheOperatorImageTag = 'latest';
            }
            else {
                // The colon separates image name from the tag
                deployedCheOperatorImageName = beforeLastColon;
                deployedCheOperatorImageTag = afterLastColon;
            }
        }
    }
    return [deployedCheOperatorImageName, deployedCheOperatorImageTag];
}
exports.getImageNameAndTag = getImageNameAndTag;
/**
 * Returns the tag of the image.
 */
function getImageTag(image) {
    let entries = image.split('@');
    if (entries.length === 2) {
        // digest
        return entries[1];
    }
    entries = image.split(':');
    // tag
    return entries[1];
}
exports.getImageTag = getImageTag;
function sleep(ms) {
    // tslint:disable-next-line no-string-based-set-timeout
    return new Promise(resolve => setTimeout(resolve, ms));
}
exports.sleep = sleep;
/**
 * Returns CR file content. Throws an error, if file doesn't exist.
 * @param flags - parent command flags
 * @param CRKey - key for CR file flag
 * @param command - parent command
 */
function readCRFile(flags, CRKey) {
    const CRFilePath = flags[CRKey];
    if (!CRFilePath) {
        return;
    }
    if (fs.existsSync(CRFilePath)) {
        return safeLoadFromYamlFile(CRFilePath);
    }
    throw new Error(`Unable to find file defined in the flag '--${CRKey}'`);
}
exports.readCRFile = readCRFile;
/**
 * Returns command success message with execution time.
 */
function getCommandSuccessMessage() {
    const ctx = context_1.ChectlContext.get();
    if (ctx[context_1.ChectlContext.START_TIME]) {
        if (!ctx[context_1.ChectlContext.END_TIME]) {
            ctx[context_1.ChectlContext.END_TIME] = Date.now();
        }
        const workingTimeInSeconds = Math.round((ctx[context_1.ChectlContext.END_TIME] - ctx[context_1.ChectlContext.START_TIME]) / 1000);
        const minutes = Math.floor(workingTimeInSeconds / 60);
        const seconds = (workingTimeInSeconds - minutes * 60) % 60;
        const minutesToStr = minutes.toLocaleString([], { minimumIntegerDigits: 2 });
        const secondsToStr = seconds.toLocaleString([], { minimumIntegerDigits: 2 });
        return `Command ${ctx[context_1.ChectlContext.COMMAND_ID]} has completed successfully in ${minutesToStr}:${secondsToStr}.`;
    }
    return `Command ${ctx[context_1.ChectlContext.COMMAND_ID]} has completed successfully.`;
}
exports.getCommandSuccessMessage = getCommandSuccessMessage;
/**
 * Wraps error into command error.
 */
function wrapCommandError(error) {
    const ctx = context_1.ChectlContext.get();
    const logDirectory = ctx[context_1.ChectlContext.LOGS_DIR];
    let commandErrorMessage = `Command ${ctx[context_1.ChectlContext.COMMAND_ID]} failed. Error log: ${ctx[context_1.ChectlContext.ERROR_LOG]}.`;
    if (logDirectory && isDirEmpty(logDirectory)) {
        commandErrorMessage += ` CodeReady Workspaces logs: ${logDirectory}.`;
    }
    return newError(commandErrorMessage, error);
}
exports.wrapCommandError = wrapCommandError;
function newError(message, cause) {
    const error = new Error(message);
    error.stack += `\nCause: ${cause.stack}`;
    return error;
}
exports.newError = newError;
function notifyCommandCompletedSuccessfully() {
    notifier.notify({
        title: 'crwctl',
        message: getCommandSuccessMessage(),
    });
}
exports.notifyCommandCompletedSuccessfully = notifyCommandCompletedSuccessfully;
function askForChectlUpdateIfNeeded() {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const ctx = context_1.ChectlContext.get();
        if (yield version_1.VersionHelper.isChectlUpdateAvailable(ctx[context_1.ChectlContext.CACHE_DIR])) {
            cli_ux_1.cli.info('A more recent version of crwctl is available. To deploy the latest version of CodeReady Workspaces, update the crwctl tool first.');
            if (yield cli_ux_1.cli.confirm('Do you want to update crwctl now? [y/n]')) {
                // Update crwctl
                yield update_1.default.run([]);
                cli_ux_1.cli.exit(0);
            }
        }
    });
}
exports.askForChectlUpdateIfNeeded = askForChectlUpdateIfNeeded;
/**
 * Determine if a directory is empty.
 */
function isDirEmpty(dirname) {
    try {
        return fs.readdirSync(dirname).length === 0;
        // Fails in case if directory doesn't exist
    }
    catch (_a) {
        return true;
    }
}
exports.isDirEmpty = isDirEmpty;
/**
 * Returns current crwctl version defined in package.json.
 */
function getProjectVersion() {
    return pkjson.version;
}
exports.getProjectVersion = getProjectVersion;
/**
 * Returns current crwctl version defined in package.json.
 */
function getProjectName() {
    return pkjson.name;
}
exports.getProjectName = getProjectName;
function readPackageJson() {
    return JSON.parse(fs.readFileSync('../package.json').toString());
}
exports.readPackageJson = readPackageJson;
function safeLoadFromYamlFile(filePath) {
    return yaml.load(fs.readFileSync(filePath).toString());
}
exports.safeLoadFromYamlFile = safeLoadFromYamlFile;
function safeSaveYamlToFile(yamlObject, filePath) {
    fs.writeFileSync(filePath, yaml.dump(yamlObject));
}
exports.safeSaveYamlToFile = safeSaveYamlToFile;
function downloadFile(url, dest) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const streamWriter = fs.createWriteStream(dest);
        const response = yield axios_1.default({ url, method: 'GET', responseType: 'stream' });
        response.data.pipe(streamWriter);
        return new Promise((resolve, reject) => {
            streamWriter.on('finish', resolve);
            streamWriter.on('error', reject);
        });
    });
}
exports.downloadFile = downloadFile;
/**
 * Downloads yaml file and returns data converted to JSON.
 * @param url link to yaml file
 */
function downloadYaml(url) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const axiosInstance = axios_1.default.create({
            httpsAgent: new https.Agent({}),
        });
        const response = yield axiosInstance.get(url);
        return yaml.load(response.data);
    });
}
exports.downloadYaml = downloadYaml;
function getEmbeddedTemplatesDirectory() {
    // Embedded templates are located in the templates directory that is in the project/installation root:
    // crwctl
    //  |- templates
    //  |- src
    //  |   |- util.ts
    //  |  ...
    //  |- lib
    //  |   |- util.js
    // ... ...
    // __dirname is
    //   project_root/src if dev mode,
    //   installation_root/lib if run from an installed location
    return path.join(__dirname, '..', 'templates');
}
exports.getEmbeddedTemplatesDirectory = getEmbeddedTemplatesDirectory;
/**
 * The default CodeReady Workspaces namespace has been changed from 'che' to 'eclipse-che'.
 * It checks if legacy namespace 'che' exists. If so all crwctl commands
 * will launched against that namespace otherwise default 'eclipse-che' namespace will be used.
 */
function findWorkingNamespace(flags) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (flags.chenamespace) {
            // use user specified namespace
            return flags.chenamespace;
        }
        const kubeHelper = new kube_1.KubeHelper(flags);
        if (yield kubeHelper.getNamespace(constants_1.DEFAULT_CHE_NAMESPACE)) {
            return constants_1.DEFAULT_CHE_NAMESPACE;
        }
        if (yield kubeHelper.getNamespace(constants_1.LEGACY_CHE_NAMESPACE)) {
            return constants_1.LEGACY_CHE_NAMESPACE;
        }
        return constants_1.DEFAULT_CHE_NAMESPACE;
    });
}
exports.findWorkingNamespace = findWorkingNamespace;
/**
 * Return linux distribution if crwctl command is executed in linux
 */
function getDistribution() {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (os.platform() === 'linux') {
            try {
                const platorm = yield util_1.promisify(getos)();
                return platorm.dist;
            }
            catch (_a) {
                return;
            }
        }
        return;
    });
}
exports.getDistribution = getDistribution;
function addTrailingSlash(url) {
    if (url.endsWith('/')) {
        return url;
    }
    return url + '/';
}
exports.addTrailingSlash = addTrailingSlash;
//# sourceMappingURL=util.js.map
